/*****************************************************************************************************************************
--
--    File Name    : Mux2x1.v

--    Description  : This module implements Alpha 2:1 MUX
--                                  

-- Targeted device : Microsemi-SoC                     
-- Author          : India Solutions Team

-- SVN Revision Information:
-- SVN $Revision: TBD
-- SVN $Date: TBD
--
--
--
-- COPYRIGHT 2016 BY MICROSEMI 
-- THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
-- FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
-- MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
-- NO BACK-UP OF THE FILE SHOULD BE MADE. 
-- 

--******************************************************************************************************************************/


module Mux2x1 #(
/******************************************************************************************************************************/
/***********************************************Parameters*********************************************************************/
/******************************************************************************************************************************/
                parameter g_DATA_BITWIDTH = 8 //Width of input/output data
               )
              (
/******************************************************************************************************************************/
/***********************************************Module IO**********************************************************************/
/******************************************************************************************************************************/
               //Clock and reset
               input wire SYS_CLK_I,
               input wire nRESET_I,
               //Input Data ports
               input wire[g_DATA_BITWIDTH - 1 : 0] DATA00_In_i,
               input wire[g_DATA_BITWIDTH - 1 : 0] DATA01_In_i,
               input wire[g_DATA_BITWIDTH - 1 : 0] DATA02_In_i,
               input wire DATAIn0_VLD_i,
               input wire[g_DATA_BITWIDTH - 1 : 0] DATA10_In_i,
               input wire[g_DATA_BITWIDTH - 1 : 0] DATA11_In_i,
               input wire[g_DATA_BITWIDTH - 1 : 0] DATA12_In_i,
               input wire DATAIn1_VLD_i,
               input wire SEL_i,               
               //Output data ports 
               output reg[g_DATA_BITWIDTH - 1 : 0] DATA0_OUT_o,
               output reg[g_DATA_BITWIDTH - 1 : 0] DATA1_OUT_o,
               output reg[g_DATA_BITWIDTH - 1 : 0] DATA2_OUT_o,
               output reg DATAOut_VLD_o
               
              );


  //Registers declaration.
 reg[g_DATA_BITWIDTH - 1 : 0] DATA00_In_i_d;
 reg[g_DATA_BITWIDTH - 1 : 0] DATA01_In_i_d;
 reg[g_DATA_BITWIDTH - 1 : 0] DATA02_In_i_d;
 reg DATAIn0_VLD_i_d;
 reg[g_DATA_BITWIDTH - 1 : 0] DATA10_In_i_d;
 reg[g_DATA_BITWIDTH - 1 : 0] DATA11_In_i_d;
 reg[g_DATA_BITWIDTH - 1 : 0] DATA12_In_i_d;
 reg DATAIn1_VLD_i_d;

/******************************************************************************************************************************/
/********************************Process to register all the inputs************************************************************/
/******************************************************************************************************************************/

  always@(posedge(SYS_CLK_I),negedge(nRESET_I))
  begin
     if (!nRESET_I) begin
       DATA10_In_i_d <= {g_DATA_BITWIDTH{1'b0}};
       DATA11_In_i_d <= {g_DATA_BITWIDTH{1'b0}};    
       DATA12_In_i_d <= {g_DATA_BITWIDTH{1'b0}};
       DATA00_In_i_d <= {g_DATA_BITWIDTH{1'b0}};
       DATA01_In_i_d <= {g_DATA_BITWIDTH{1'b0}};    
       DATA02_In_i_d <= {g_DATA_BITWIDTH{1'b0}};
       DATAIn0_VLD_i_d <= 1'b0;
       DATAIn1_VLD_i_d <= 1'b0;
      end
     else begin
           DATA10_In_i_d <= DATA10_In_i;
           DATA11_In_i_d <= DATA11_In_i;    
           DATA12_In_i_d <= DATA12_In_i;
           DATA00_In_i_d <= DATA00_In_i;
           DATA01_In_i_d <= DATA01_In_i;    
           DATA02_In_i_d <= DATA02_In_i;
           DATAIn0_VLD_i_d <= DATAIn0_VLD_i;
           DATAIn1_VLD_i_d <= DATAIn1_VLD_i;
       end 
  end


/******************************************************************************************************************************/
/********************************Process to select the data depending on Demo Selection****************************************/
/******************************************************************************************************************************/

  always@(posedge(SYS_CLK_I),negedge(nRESET_I))
  begin
     if (!nRESET_I) begin
       DATA0_OUT_o <= {g_DATA_BITWIDTH{1'b0}};
       DATA1_OUT_o <= {g_DATA_BITWIDTH{1'b0}};    
       DATA2_OUT_o <= {g_DATA_BITWIDTH{1'b0}};
     end
     else begin
       if ( SEL_i == 1'b1) begin
           DATA0_OUT_o <= DATA10_In_i_d;
           DATA1_OUT_o <= DATA11_In_i_d;    
           DATA2_OUT_o <= DATA12_In_i_d;
       end
      else begin
           DATA0_OUT_o <= DATA00_In_i_d;
           DATA1_OUT_o <= DATA01_In_i_d;    
           DATA2_OUT_o <= DATA02_In_i_d;
       end
     end     
  end


/******************************************************************************************************************************/
/********************************Process to select the data valid depending on Demo Selection**********************************/
/******************************************************************************************************************************/
  always@(posedge(SYS_CLK_I),negedge(nRESET_I))
  begin
     if (!nRESET_I) begin
       DATAOut_VLD_o <= 1'b0;
     end
     else begin
       if (SEL_i == 1'b1) begin
           DATAOut_VLD_o <= DATAIn1_VLD_i_d;
       end
       else begin
            DATAOut_VLD_o <= DATAIn0_VLD_i_d;
       end
    end     
  end


 
  
endmodule

